@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_FILE_ALLOCATION_ENTRY_H_
-#define _D_FILE_ALLOCATION_ENTRY_H_
+#ifndef D_FILE_ALLOCATION_ENTRY_H
+#define D_FILE_ALLOCATION_ENTRY_H
 
 #include "RequestGroupEntry.h"
 #include "ProgressAwareEntry.h"
@@ -70,4 +70,4 @@ public:
 
 } // namespace aria2
 
-#endif // _D_FILE_ALLOCATION_ENTRY_H_
+#endif // D_FILE_ALLOCATION_ENTRY_H
