@@ -33,30 +33,23 @@
  */
 /* copyright --> */
 #include "RequestSlot.h"
+#include "wallclock.h"
 
 namespace aria2 {
 
 RequestSlot RequestSlot::nullSlot = RequestSlot();
 
-void RequestSlot::setDispatchedTime() {
-  dispatchedTime.reset();
+void RequestSlot::setDispatchedTime(time_t sec) {
+  dispatchedTime_.reset(sec);
 }
 
-void RequestSlot::setDispatchedTime(time_t secFromEpoch) {
-  dispatchedTime.setTimeInSec(secFromEpoch);
-}
-
-bool RequestSlot::isTimeout(const struct timeval& now, time_t timeoutSec) const {
-  return dispatchedTime.difference(now) >= timeoutSec;
-}
-
-unsigned int RequestSlot::getLatencyInMillis() const {
-  return dispatchedTime.differenceInMillis();
+bool RequestSlot::isTimeout(time_t timeoutSec) const {
+  return dispatchedTime_.difference(global::wallclock) >= timeoutSec;
 }
 
 bool RequestSlot::isNull(const RequestSlot& requestSlot) {
-  return requestSlot.index == 0 && requestSlot.begin == 0&&
-    requestSlot.length == 0;
+  return requestSlot.index_ == 0 && requestSlot.begin_ == 0&&
+    requestSlot.length_ == 0;
 }
 
 } // namespace aria2
