@@ -1,14 +1,19 @@
 #include "BtHaveNoneMessage.h"
-#include "PeerMessageUtil.h"
+
+#include <cstring>
+
 #include <cppunit/extensions/HelperMacros.h>
 
-using namespace std;
+#include "bittorrent_helper.h"
+#include "Peer.h"
+
+namespace aria2 {
 
 class BtHaveNoneMessageTest:public CppUnit::TestFixture {
 
   CPPUNIT_TEST_SUITE(BtHaveNoneMessageTest);
   CPPUNIT_TEST(testCreate);
-  CPPUNIT_TEST(testGetMessage);
+  CPPUNIT_TEST(testCreateMessage);
   CPPUNIT_TEST(testDoReceivedAction);
   CPPUNIT_TEST(testToString);
   CPPUNIT_TEST_SUITE_END();
@@ -19,7 +24,7 @@ public:
   }
 
   void testCreate();
-  void testGetMessage();
+  void testCreateMessage();
   void testDoReceivedAction();
   void testToString();
 };
@@ -29,14 +34,14 @@ CPPUNIT_TEST_SUITE_REGISTRATION(BtHaveNoneMessageTest);
 
 void BtHaveNoneMessageTest::testCreate() {
   unsigned char msg[5];
-  PeerMessageUtil::createPeerMessageString(msg, sizeof(msg), 1, 15);
-  BtHaveNoneMessageHandle pm = BtHaveNoneMessage::create(&msg[4], 1);
-  CPPUNIT_ASSERT_EQUAL((int8_t)15, pm->getId());
+  bittorrent::createPeerMessageString(msg, sizeof(msg), 1, 15);
+  SharedHandle<BtHaveNoneMessage> pm = BtHaveNoneMessage::create(&msg[4], 1);
+  CPPUNIT_ASSERT_EQUAL((uint8_t)15, pm->getId());
 
   // case: payload size is wrong
   try {
     unsigned char msg[6];
-    PeerMessageUtil::createPeerMessageString(msg, sizeof(msg), 2, 15);
+    bittorrent::createPeerMessageString(msg, sizeof(msg), 2, 15);
     BtHaveNoneMessage::create(&msg[4], 2);
     CPPUNIT_FAIL("exception must be thrown.");
   } catch(...) {
@@ -44,23 +49,26 @@ void BtHaveNoneMessageTest::testCreate() {
   // case: id is wrong
   try {
     unsigned char msg[5];
-    PeerMessageUtil::createPeerMessageString(msg, sizeof(msg), 1, 16);
+    bittorrent::createPeerMessageString(msg, sizeof(msg), 1, 16);
     BtHaveNoneMessage::create(&msg[4], 1);
     CPPUNIT_FAIL("exception must be thrown.");
   } catch(...) {
   }
 }
 
-void BtHaveNoneMessageTest::testGetMessage() {
+void BtHaveNoneMessageTest::testCreateMessage() {
   BtHaveNoneMessage msg;
   unsigned char data[5];
-  PeerMessageUtil::createPeerMessageString(data, sizeof(data), 1, 15);
-  CPPUNIT_ASSERT(memcmp(msg.getMessage(), data, 5) == 0);
+  bittorrent::createPeerMessageString(data, sizeof(data), 1, 15);
+  unsigned char* rawmsg = msg.createMessage();
+  CPPUNIT_ASSERT(memcmp(rawmsg, data, 5) == 0);
+  delete [] rawmsg;
 }
 
 void BtHaveNoneMessageTest::testDoReceivedAction() {
   BtHaveNoneMessage msg;
-  PeerHandle peer = new Peer("host", 6969, 16*1024, 256*1024);
+  SharedHandle<Peer> peer(new Peer("host", 6969));
+  peer->allocateSessionResource(1024, 1024*1024);
   peer->setFastExtensionEnabled(true);
   msg.setPeer(peer);
   msg.doReceivedAction();
@@ -74,5 +82,7 @@ void BtHaveNoneMessageTest::testDoReceivedAction() {
 
 void BtHaveNoneMessageTest::testToString() {
   BtHaveNoneMessage msg;
-  CPPUNIT_ASSERT_EQUAL(string("have none"), msg.toString());
+  CPPUNIT_ASSERT_EQUAL(std::string("have none"), msg.toString());
 }
+
+} // namespace aria2
