@@ -21,7 +21,7 @@ class UriListParserTest : public CppUnit::TestFixture {
   CPPUNIT_TEST(testHasNext);
   CPPUNIT_TEST_SUITE_END();
 private:
-  std::string list2String(const std::deque<std::string>& src);
+  std::string list2String(const std::vector<std::string>& src);
 public:
   void setUp() {
   }
@@ -32,20 +32,20 @@ public:
 
 CPPUNIT_TEST_SUITE_REGISTRATION( UriListParserTest );
 
-std::string UriListParserTest::list2String(const std::deque<std::string>& src)
+std::string UriListParserTest::list2String(const std::vector<std::string>& src)
 {
   std::ostringstream strm;
   std::copy(src.begin(), src.end(), std::ostream_iterator<std::string>(strm, " "));
-  return util::trim(strm.str());
+  return util::strip(strm.str());
 }
 
 void UriListParserTest::testHasNext()
 {
-  std::ifstream in("filelist1.txt", std::ios::binary);
+  std::ifstream in(A2_TEST_DIR"/filelist1.txt", std::ios::binary);
 
   UriListParser flp(in);
 
-  std::deque<std::string> uris;
+  std::vector<std::string> uris;
   Option reqOp;
 
   CPPUNIT_ASSERT(flp.hasNext());
@@ -62,7 +62,7 @@ void UriListParserTest::testHasNext()
 
   flp.parseNext(uris, reqOp);
   CPPUNIT_ASSERT_EQUAL(std::string("ftp://localhost/aria2.tar.bz2"),
-		       list2String(uris));
+                       list2String(uris));
   CPPUNIT_ASSERT_EQUAL(std::string("/tmp"), reqOp.get(PREF_DIR));
   CPPUNIT_ASSERT_EQUAL(std::string("chunky_chocolate"), reqOp.get(PREF_OUT));
 
